@@ -1,15 +1,16 @@
 import numpy as np
 
+from shapely import ops
 from shapely.geometry import Polygon
 
-from collections import deque
-
 from .. import util
 from .. import bounds
 from .. import graph
+from .. import geometry
+from .. import grouping
 
-from ..constants import tol_path as tol
 from ..constants import log
+from ..constants import tol_path as tol
 from ..transformations import transform_points
 
 from .simplify import fit_circle_check
@@ -83,14 +84,11 @@ def enclosure_tree(polygons):
     # a root or exterior curve has an even number of parents
     # wrap in dict call to avoid networkx view
     degree = dict(contains.in_degree())
-
     # convert keys and values to numpy arrays
     indexes = np.array(list(degree.keys()))
     degrees = np.array(list(degree.values()))
-
     # roots are curves with an even inward degree (parent count)
     roots = indexes[(degrees % 2) == 0]
-
     # if there are multiple nested polygons split the graph
     # so the contains logic returns the individual polygons
     if len(degrees) > 0 and degrees.max() > 1:
@@ -228,7 +226,7 @@ def transform_polygon(polygon, matrix):
     return result
 
 
-def plot_polygon(polygon, show=True, **kwargs):
+def plot(polygon, show=True, **kwargs):
     """
     Plot a shapely polygon using matplotlib.
 
@@ -247,7 +245,7 @@ def plot_polygon(polygon, show=True, **kwargs):
         plt.plot(*single.exterior.xy, **kwargs)
         for interior in single.interiors:
             plt.plot(*interior.xy, **kwargs)
-    # make aspect ratio non- stupid
+    # make aspect ratio non-stupid
     plt.axes().set_aspect('equal', 'datalim')
     if util.is_sequence(polygon):
         [plot_single(i) for i in polygon]
@@ -288,10 +286,10 @@ def resample_boundaries(polygon, resolution, clip=None):
         clip = [8, 200]
     # create a sequence of [(n,2)] points
     kwargs = {'shell': resample_boundary(polygon.exterior),
-              'holes': deque()}
+              'holes': []}
     for interior in polygon.interiors:
         kwargs['holes'].append(resample_boundary(interior))
-    kwargs['holes'] = np.array(kwargs['holes'])
+
     return kwargs
 
 
@@ -544,22 +542,29 @@ def sample(polygon, count, factor=1.5, max_iter=10):
     bounds = np.reshape(polygon.bounds, (2, 2))
     extents = bounds.ptp(axis=0)
 
-    hit = []
-    hit_count = 0
+    # how many points to check per loop iteration
     per_loop = int(count * factor)
 
+    # start with some rejection sampling
+    points = bounds[0] + extents * np.random.random((per_loop, 2))
+    # do the point in polygon test and append resulting hits
+    mask = vectorized.contains(polygon, *points.T)
+    hit = [points[mask]]
+    hit_count = len(hit[0])
+    # if our first non-looping check got enough samples exit
+    if hit_count >= count:
+        return hit[0][:count]
+
+    # if we have to do iterations loop here slowly
     for i in range(max_iter):
         # generate points inside polygons AABB
         points = np.random.random((per_loop, 2))
         points = (points * extents) + bounds[0]
-
         # do the point in polygon test and append resulting hits
         mask = vectorized.contains(polygon, *points.T)
         hit.append(points[mask])
-
         # keep track of how many points we've collected
         hit_count += len(hit[-1])
-
         # if we have enough points exit the loop
         if hit_count > count:
             break
@@ -658,3 +663,127 @@ def repair_invalid(polygon, scale=None, rtol=.5):
         return buffered
 
     raise ValueError('unable to recover polygon!')
+
+
+def projected(mesh,
+              normal,
+              origin=None,
+              pad=1e-5,
+              tol_dot=0.01,
+              max_regions=200):
+    """
+    Project a mesh onto a plane and then extract the polygon
+    that outlines the mesh projection on that plane.
+
+    Parameters
+    ----------
+    mesh : trimesh.Trimesh
+      Source geometry
+    check : bool
+      If True make sure is flat
+    normal : (3,) float
+      Normal to extract flat pattern along
+    origin : None or (3,) float
+      Origin of plane to project mesh onto
+    pad : float
+      Proportion to pad polygons by before unioning
+      and then de-padding result by to avoid zero-width gaps.
+    tol_dot : float
+      Tolerance for discarding on-edge triangles.
+    max_regions : int
+      Raise an exception if the mesh has more than this
+      number of disconnected regions to fail quickly before unioning.
+
+    Returns
+    ----------
+    projected : shapely.geometry.Polygon
+      Outline of source mesh
+
+    Raises
+    ---------
+    ValueError
+      If max_regions is exceeded
+    """
+    # make sure normal is a unitized copy
+    normal = np.array(normal, dtype=np.float64)
+    normal /= np.linalg.norm(normal)
+
+    # the projection of each face normal onto facet normal
+    dot_face = np.dot(normal, mesh.face_normals.T)
+    # check if face lies on front or back of normal
+    front = dot_face > tol_dot
+    back = dot_face < -tol_dot
+    # divide the mesh into front facing section and back facing parts
+    # and discard the faces perpendicular to the axis.
+    # since we are doing a unary_union later we can use the front *or*
+    # the back so we use which ever one has fewer triangles
+    # we want the largest nonzero group
+    count = np.array([front.sum(), back.sum()])
+    if count.min() == 0:
+        # if one of the sides has zero faces we need the other
+        pick = count.argmax()
+    else:
+        # otherwise use the normal direction with the fewest faces
+        pick = count.argmin()
+    # use the picked side
+    side = [front, back][pick]
+
+    # subset the adjacency pairs to ones which have both faces included
+    # on the side we are currently looking at
+    adjacency_check = side[mesh.face_adjacency].all(axis=1)
+    adjacency = mesh.face_adjacency[adjacency_check]
+
+    # a sequence of face indexes that are connected
+    face_groups = graph.connected_components(
+        adjacency, nodes=np.nonzero(side)[0])
+
+    # if something is goofy we may end up with thousands of
+    # regions that do nothing except hang for an hour then segfault
+    if len(face_groups) > max_regions:
+        raise ValueError('too many disconnected groups!')
+
+    # reshape edges into shape length of faces for indexing
+    edges = mesh.edges_sorted.reshape((-1, 6))
+    # transform from the mesh frame in 3D to the XY plane
+    to_2D = geometry.plane_transform(
+        origin=origin, normal=normal)
+    # transform mesh vertices to 2D and clip the zero Z
+    vertices_2D = transform_points(
+        mesh.vertices, to_2D)[:, :2]
+
+    polygons = []
+    for faces in face_groups:
+        # index edges by face then shape back to individual edges
+        edge = edges[faces].reshape((-1, 2))
+        # edges that occur only once are on the boundary
+        group = grouping.group_rows(edge, require_count=1)
+        # turn each region into polygons
+        polygons.extend(edges_to_polygons(
+            edges=edge[group], vertices=vertices_2D))
+
+    # some types of errors will lead to a bajillion disconnected
+    # regions and the union will take forever to fail
+    # so exit here early
+    if len(polygons) > max_regions:
+        raise ValueError('too many disconnected groups!')
+
+    # if there is only one region we don't need to run a union
+    elif len(polygons) == 1:
+        polygon = polygons[0]
+        # we do however need to double buffer to de-garbage the polygon
+        scale = np.reshape(polygon.bounds, (2, 2)).ptp(axis=0).max()
+        padding = scale * pad
+        polygon = polygon.buffer(padding).buffer(-padding)
+    else:
+        # get all points for every AABB
+        extrema = np.reshape([p.bounds for p in polygons], (-1, 2))
+        # extract the model scale from the maximum AABB side length
+        scale = extrema.ptp(axis=0).max()
+        # pad each polygon proportionally to that scale
+        distance = abs(scale * pad)
+        # inflate each polygon before unioning to remove zero-size
+        # gaps then deflate the result after unioning by the same amount
+        polygon = ops.unary_union(
+            [p.buffer(distance) for p in polygons]).buffer(-distance)
+
+    return polygon
